set(DOXYGEN_FOUND AUTO CACHE STRING "Enable Doxygen documentation generator")
set_property(CACHE DOXYGEN_FOUND
             PROPERTY STRINGS
                      ON
                      OFF
                      AUTO)

find_package(Doxygen)
if(DOXYGEN_FOUND)
  set(DOXYGEN_FOUND ON CACHE STRING "Doxygen documentation generator enabled" FORCE)
else()
  set(DOXYGEN_FOUND OFF CACHE STRING "Doxygen documentation generator disabled" FORCE)
endif()

function(add_doxygen_target)
  set(options)
  set(oneValueArgs
      TARGET_NAME
      DOXY_FILE
      OUTPUT_DIRECTORY
      LATEX
      HTML)
  set(multiValueArgs)

  cmake_parse_arguments(_DOXYGEN
                        "${options}"
                        "${oneValueArgs}"
                        "${multiValueArgs}"
                        ${ARGN})
  if(DOXYGEN_FOUND)
    set(GENERATE_XML NO)
    set(GENERATE_HTML NO)
    set(GENERATE_LATEX NO)
    if(_DOXYGEN_HTML)
      set(GENERATE_HTML YES)
    endif()
    if(_DOXYGEN_LATEX)
      set(GENERATE_LATEX YES)
    endif()

    configure_file(${_DOXYGEN_DOXY_FILE} ${_DOXYGEN_OUTPUT_DIRECTORY}/Doxyfile @ONLY)
    message("Doxygen build started")

    add_custom_target(${_DOXYGEN_TARGET_NAME} VERBATIM
                      COMMAND ${DOXYGEN_EXECUTABLE} ${_DOXYGEN_OUTPUT_DIRECTORY}/Doxyfile
                      WORKING_DIRECTORY ${_DOXYGEN_OUTPUT_DIRECTORY}
                      COMMENT "Generating API documentation with Doxygen")
  else()
    add_custom_target(${_DOXYGEN_TARGET_NAME} VERBATIM
                      COMMENT "Doxygen is not found. Skipping target ${_DOXYGEN_TARGET_NAME} build")
  endif()
endfunction()
